{
  description = "Gargantext PureScript fronetnd";

  inputs.flake-utils.url = "github:numtide/flake-utils";

  # bun-1.1
  inputs.nixpkgs.url = "nixpkgs/860e65d27036476edfb85dd847d982277880b143";

  # bun-1.0
  #inputs.nixpkgs.url = "nixpkgs/23.11";

  inputs.purescript-overlay = {
    url = "github:thomashoneyman/purescript-overlay";
    inputs.nixpkgs.follows = "nixpkgs";
  };
  
  outputs = { self, nixpkgs, flake-utils, purescript-overlay }:
    flake-utils.lib.eachDefaultSystem (system:
      #let pkgs = nixpkgs.legacyPackages.${system};
      let
        pkgs = import nixpkgs {
          inherit system;
          overlays = [ purescript-overlay.overlays.default ];
        };

        dependencies = with pkgs; [
          purescript
          spago-unstable
          bun
          sqlite
          nodejs
        ];
    
      serve = pkgs.writeShellScriptBin "serve" ''
        #!/usr/bin/env bash
        set -e
    
        #yarn server
        bun run server
    
      '';
    
      build-watch = pkgs.writeShellScriptBin "build-watch" ''
        #!/usr/bin/env bash
        set -e
    
        echo "Build watch"
        bun spago build -w --then browserify
      '';
    
      build-zephyr = pkgs.writeShellScriptBin "build-zephyr" ''
        #!/usr/bin/env bash
        set -e
    
        bun spago build --purs-args '--codegen corefn,js'
        zephyr -f Main.main
        browserify-zephyr
      '';
    
      minify-bundle = pkgs.writeShellScriptBin "minify-bundle" ''
        #!/usr/bin/env bash
        set -e
    
        bun run minify
      '';
    
      in
        {
          packages = {
            # sometimes bun is broken because of better-sqlite3
            fix-bun = pkgs.writeShellApplication {
              name = "fix-bun";
              runtimeInputs = dependencies;
              text = ''
              #!${pkgs.stdenv.shell}
              set -e

              DIR=./node_modules/better-sqlite3

              if [ -d "$DIR" ]; then
                cd $DIR
                ${pkgs.bun}/bin/bun install
              fi
              '';
            };
            compile = pkgs.writeShellApplication {
              name = "compile";
              runtimeInputs = dependencies;
              text = ''
              #!${pkgs.stdenv.shell}
              set -e

              echo "Installing JS Dependencies"
              #yarn
              ${pkgs.bun}/bin/bun install
              ${pkgs.bun}/bin/bun install --dev
          
              echo "Compiling"
              ${pkgs.bun}/bin/bun run build
              '';
            };
            build = pkgs.writeShellApplication {
              name = "build";
              runtimeInputs = dependencies;
              text = ''
              #!${pkgs.stdenv.shell}
              set -e

              ${self.packages.${system}.compile}/bin/compile

              echo "Bundling"
              ${pkgs.bun}/bin/bun run bundle              
              '';
            };
            test-ps = pkgs.writeShellApplication {
              name = "test-ps";
              runtimeInputs = dependencies;
              text = ''
              #!${pkgs.stdenv.shell}
              set -e
          
              ${self.packages.${system}.compile}/bin/compile
          
              echo "Testing"
              ${pkgs.bun}/bin/bun run test
            '';
            };    
            repl = pkgs.writeShellApplication {
              name = "repl";
              runtimeInputs = dependencies;
              text = ''
              #!${pkgs.stdenv.shell}
          
              ${pkgs.bun}/bin/bun run repl
            '';
            };
            
            build-css = pkgs.writeShellApplication {
              name = "build-css";
              runtimeInputs = dependencies;
              text = ''
              #!${pkgs.stdenv.shell}
              set -e
          
              #yarn css
              ${pkgs.bun}/bin/bun run css
            '';
            };
          };
          
          devShells.default = pkgs.mkShell {
            name = "purescript-gargantext";
            #inputsFrom = builtins.attrValues self.packages.${system};
            buildInputs = with pkgs; (dependencies ++ [
              # scripts
              self.packages.${system}.build
              self.packages.${system}.compile
              self.packages.${system}.fix-bun
              self.packages.${system}.test-ps
              self.packages.${system}.repl
              
              self.packages.${system}.build-css
              build-watch
              build-zephyr
              minify-bundle
              serve
            ]);
          };
        }
    );
}
